pub const HTML_PAGE: &str = r#"
<html lang="en">
<head>
    <meta charset="UTF - 8">
    <meta name="viewport" content="width = device-width, initial-scale=1.0">
    <title>Sonic Search</title>
</head>
<body>
<h1>Sonic Search Interface</h1>
<div>
    <span id="offset" style="display: none">0</span>
    <h2>Search</h2>
    <select id="lists"></select>
    <input type="number" id="limit" placeholder="Limit" value="10" min="0" style="width: 80px;">
    <input type="text" id="query" placeholder="Search Query">
    <button onclick="searchDocuments()">Search</button>
    <p id="noItemsFound" style="display: none;">no items found</p>
    <ol id="results"></ol>
    <button onclick="loadMore()" id="showMore" style="display: none;">More</button>
</div>

<script>
    document.addEventListener("DOMContentLoaded", get_lists);

    async function get_lists() {
        let lists_object = document.getElementById("lists")
        lists_object.innerHTML = ""

        let lists = await fetch(`/get-lists`)
        lists = await lists.json()

        for (let list of lists) {
            let option = document.createElement("option")
            option.innerText = list
            lists_object.appendChild(option)
        }
    }

    async function searchDocuments() {
        document.getElementById("offset").innerText = '0';
        document.getElementById("showMore").style.display = "block";
        
        const query = document.getElementById('query').value;
        const lists = document.getElementById("lists");
        const limit = document.getElementById("limit").value;
        const noItemsFound = document.getElementById("noItemsFound");
        
        noItemsFound.style.display = "none";

        const response = await fetch(`/search?q=${encodeURIComponent(query)}&list=${encodeURIComponent(lists.value)}&limit=${encodeURIComponent(limit)}`);
        const results = await response.json();
        const resultsList = document.getElementById('results');
        resultsList.innerHTML = '';
        results.forEach(result => {
            const li = document.createElement('li');
            li.textContent = result;
            resultsList.appendChild(li);
        });

        if (results.length === 0) {
            noItemsFound.style.display = "block";
        }
        
        const offset = document.getElementById("offset");
        offset.innerText =  (parseInt(offset.innerText) + results.length).toString();

        if (results.length < limit){
            document.getElementById("showMore").style.display = "none";
        }
    }
    
    async function loadMore(){
        const query = document.getElementById('query').value;
        const lists = document.getElementById("lists");
        const limit = document.getElementById("limit").value;
        const offset = document.getElementById("offset");
        
        const response = await fetch(`/search?q=${encodeURIComponent(query)}&list=${encodeURIComponent(lists.value)}&limit=${encodeURIComponent(limit)}&offset=${encodeURIComponent(offset.innerText)}`);
        const results = await response.json();
        const resultsList = document.getElementById('results');
        results.forEach(result => {
            const li = document.createElement('li');
            li.textContent = result;
            resultsList.appendChild(li);
        });
        
        offset.innerText =  (parseInt(offset.innerText) + results.length).toString();
        
        if (results.length < limit){
            document.getElementById("showMore").style.display = "none";
        }
    }
</script>
</body>
</html>
"#;